// generated by CommonLibrary.cmake, do not edit.

/**
 * @file include/lunchbox/version.h
 * Defines version macros and class for library Lunchbox.
 */

#ifndef LUNCHBOX_VERSION_H
#define LUNCHBOX_VERSION_H

#include <lunchbox/api.h>
#include <string>

namespace lunchbox
{
    /** The current major version. */
#   define LUNCHBOX_VERSION_MAJOR 1

    /** The current minor version. */
#   define LUNCHBOX_VERSION_MINOR 17

    /** The current patch level. */
#   define LUNCHBOX_VERSION_PATCH 0

    /** The current SCM revision. */
#   define LUNCHBOX_VERSION_REVISION 0x0

    /** The current binary interface. */
#   define LUNCHBOX_VERSION_ABI 10ull

    /** The current major.minor.version as a string. */
#   define LUNCHBOX_VERSION_STRING \
    "1.17.0"

    /** The current major.minor.version.revision as a string. */
#   define LUNCHBOX_REV_STRING \
    "1.17.0.0x0"

/** True if the current version is newer than the given one. */
#   define LUNCHBOX_VERSION_GT( MAJOR, MINOR, PATCH )       \
    ( (LUNCHBOX_VERSION_MAJOR>MAJOR) ||                     \
      (LUNCHBOX_VERSION_MAJOR==MAJOR && (LUNCHBOX_VERSION_MINOR>MINOR || \
          (LUNCHBOX_VERSION_MINOR==MINOR && LUNCHBOX_VERSION_PATCH>PATCH))))

/** True if the current version is equal or newer to the given. */
#   define LUNCHBOX_VERSION_GE( MAJOR, MINOR, PATCH )       \
    ( (LUNCHBOX_VERSION_MAJOR>MAJOR) ||                     \
      (LUNCHBOX_VERSION_MAJOR==MAJOR && (LUNCHBOX_VERSION_MINOR>MINOR || \
          (LUNCHBOX_VERSION_MINOR==MINOR && LUNCHBOX_VERSION_PATCH>=PATCH))))

/** True if the current version is older than the given one. */
#   define LUNCHBOX_VERSION_LT( MAJOR, MINOR, PATCH )       \
    ( (LUNCHBOX_VERSION_MAJOR<MAJOR) ||                     \
      (LUNCHBOX_VERSION_MAJOR==MAJOR && (LUNCHBOX_VERSION_MINOR<MINOR || \
          (LUNCHBOX_VERSION_MINOR==MINOR && LUNCHBOX_VERSION_PATCH<PATCH))))

/** True if the current version is older or equal to the given. */
#   define LUNCHBOX_VERSION_LE( MAJOR, MINOR, PATCH )       \
    ( (LUNCHBOX_VERSION_MAJOR<MAJOR) ||                     \
      (LUNCHBOX_VERSION_MAJOR==MAJOR && (LUNCHBOX_VERSION_MINOR<MINOR || \
        (LUNCHBOX_VERSION_MINOR==MINOR && LUNCHBOX_VERSION_PATCH<=PATCH))))

/** Information about the current Lunchbox version. */
class LUNCHBOX_API Version
{
public:
    /** @return the current major version of Lunchbox. */
    static int getMajor();

    /** @return the current minor version of Lunchbox. */
    static int getMinor();

    /** @return the current patch level of Lunchbox. */
    static int getPatch();

    /** @return the current Lunchbox version (MM.mm.pp). */
    static std::string getString();

    /** @return the SCM revision. */
    static unsigned long long getRevision();

    /** @return the current binary interface version of Lunchbox. */
    static int getABI();

    /** @return the current Lunchbox version plus the git SHA hash (MM.mm.pp.rev). */
    static std::string getRevString();

    /**
     * Runtime check for ABI compatibility.
     *
     * Call from code using Lunchbox. Will fail if the executable
     * was compiled against a version incompatible with the runtime version.
     *
     * @return true if the link-time and compile-time DSO are compatible.
     */
    static bool check()
    {
        return getMajor()==LUNCHBOX_VERSION_MAJOR &&
               getMinor()==LUNCHBOX_VERSION_MINOR;
    }

    /** @return JSON schema describing this object. */
    static std::string getSchema()
    {
        return R"(
                {
                  "title": "Version",
                  "type": "object",
                  "description": "Version information of the running application",
                  "properties": {
                    "major": { "type": "integer" },
                    "minor": { "type": "integer" },
                    "patch": { "type": "integer" },
                    "abi": { "type": "integer" },
                    "revision": { "type": "string" }
                  }
               }
               )";
    }

    /** @return JSON-encoded data for this object. */
    static std::string toJSON()
    {
        return R"(
               {
                 "major": 1,
                 "minor": 17,
                 "patch": 0,
                 "abi": 10,
                 "revision": "0"
               }
               )";
    }
};

}

#endif
